@@ -496,8 +496,10 @@ void
 verify_test(void)
 {
 	unit_show_feature("signature verify");
+#ifdef USE_SHA1
 	verifytest_file("testdata/test_signatures.1", "20070818005004");
-#ifdef USE_DSA
+#endif
+#if defined(USE_DSA) && defined(USE_SHA1)
 	verifytest_file("testdata/test_signatures.2", "20080414005004");
 	verifytest_file("testdata/test_signatures.3", "20080416005004");
 	verifytest_file("testdata/test_signatures.4", "20080416005004");
@@ -505,17 +507,23 @@ verify_test(void)
 	verifytest_file("testdata/test_signatures.6", "20080416005004");
 	verifytest_file("testdata/test_signatures.7", "20070829144150");
 #endif /* USE_DSA */
+#ifdef USE_SHA1
 	verifytest_file("testdata/test_signatures.8", "20070829144150");
+#endif
 #if (defined(HAVE_EVP_SHA256) || defined(HAVE_NSS) || defined(HAVE_NETTLE)) && defined(USE_SHA2)
 	verifytest_file("testdata/test_sigs.rsasha256", "20070829144150");
+#  ifdef USE_SHA1
 	verifytest_file("testdata/test_sigs.sha1_and_256", "20070829144150");
+#  endif
 	verifytest_file("testdata/test_sigs.rsasha256_draft", "20090101000000");
 #endif
 #if (defined(HAVE_EVP_SHA512) || defined(HAVE_NSS) || defined(HAVE_NETTLE)) && defined(USE_SHA2)
 	verifytest_file("testdata/test_sigs.rsasha512_draft", "20070829144150");
 #endif
+#ifdef USE_SHA1
 	verifytest_file("testdata/test_sigs.hinfo", "20090107100022");
 	verifytest_file("testdata/test_sigs.revoked", "20080414005004");
+#endif
 #ifdef USE_GOST
 	if(sldns_key_EVP_load_gost_id())
 	  verifytest_file("testdata/test_sigs.gost", "20090807060504");
@@ -529,7 +537,14 @@ verify_test(void)
 	}
 	dstest_file("testdata/test_ds.sha384");
 #endif
+#ifdef USE_ED25519
+	if(dnskey_algo_id_is_supported(LDNS_ED25519)) {
+		verifytest_file("testdata/test_sigs.ed25519", "20170530140439");
+	}
+#endif
+#ifdef USE_SHA1
 	dstest_file("testdata/test_ds.sha1");
+#endif
 	nsectest();
 	nsec3_hash_test("testdata/test_nsec3_hash.1");
 }
