@@ -200,38 +200,64 @@ extern int yydebug;
     VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES = 410,
     VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES = 411,
     VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES = 412,
-    VAR_HARDEN_ALGO_DOWNGRADE = 413,
-    VAR_IP_TRANSPARENT = 414,
-    VAR_DISABLE_DNSSEC_LAME_CHECK = 415,
-    VAR_IP_RATELIMIT = 416,
-    VAR_IP_RATELIMIT_SLABS = 417,
-    VAR_IP_RATELIMIT_SIZE = 418,
-    VAR_RATELIMIT = 419,
-    VAR_RATELIMIT_SLABS = 420,
-    VAR_RATELIMIT_SIZE = 421,
-    VAR_RATELIMIT_FOR_DOMAIN = 422,
-    VAR_RATELIMIT_BELOW_DOMAIN = 423,
-    VAR_IP_RATELIMIT_FACTOR = 424,
-    VAR_RATELIMIT_FACTOR = 425,
-    VAR_CAPS_WHITELIST = 426,
-    VAR_CACHE_MAX_NEGATIVE_TTL = 427,
-    VAR_PERMIT_SMALL_HOLDDOWN = 428,
-    VAR_QNAME_MINIMISATION = 429,
-    VAR_QNAME_MINIMISATION_STRICT = 430,
-    VAR_IP_FREEBIND = 431,
-    VAR_DEFINE_TAG = 432,
-    VAR_LOCAL_ZONE_TAG = 433,
-    VAR_ACCESS_CONTROL_TAG = 434,
-    VAR_LOCAL_ZONE_OVERRIDE = 435,
-    VAR_ACCESS_CONTROL_TAG_ACTION = 436,
-    VAR_ACCESS_CONTROL_TAG_DATA = 437,
-    VAR_VIEW = 438,
-    VAR_ACCESS_CONTROL_VIEW = 439,
-    VAR_VIEW_FIRST = 440,
-    VAR_SERVE_EXPIRED = 441,
-    VAR_FAKE_DSA = 442,
-    VAR_LOG_IDENTITY = 443,
-    VAR_USE_SYSTEMD = 444
+    VAR_RESPONSE_IP_TAG = 413,
+    VAR_RESPONSE_IP = 414,
+    VAR_RESPONSE_IP_DATA = 415,
+    VAR_HARDEN_ALGO_DOWNGRADE = 416,
+    VAR_IP_TRANSPARENT = 417,
+    VAR_DISABLE_DNSSEC_LAME_CHECK = 418,
+    VAR_IP_RATELIMIT = 419,
+    VAR_IP_RATELIMIT_SLABS = 420,
+    VAR_IP_RATELIMIT_SIZE = 421,
+    VAR_RATELIMIT = 422,
+    VAR_RATELIMIT_SLABS = 423,
+    VAR_RATELIMIT_SIZE = 424,
+    VAR_RATELIMIT_FOR_DOMAIN = 425,
+    VAR_RATELIMIT_BELOW_DOMAIN = 426,
+    VAR_IP_RATELIMIT_FACTOR = 427,
+    VAR_RATELIMIT_FACTOR = 428,
+    VAR_SEND_CLIENT_SUBNET = 429,
+    VAR_CLIENT_SUBNET_ZONE = 430,
+    VAR_CLIENT_SUBNET_ALWAYS_FORWARD = 431,
+    VAR_CLIENT_SUBNET_OPCODE = 432,
+    VAR_MAX_CLIENT_SUBNET_IPV4 = 433,
+    VAR_MAX_CLIENT_SUBNET_IPV6 = 434,
+    VAR_CAPS_WHITELIST = 435,
+    VAR_CACHE_MAX_NEGATIVE_TTL = 436,
+    VAR_PERMIT_SMALL_HOLDDOWN = 437,
+    VAR_QNAME_MINIMISATION = 438,
+    VAR_QNAME_MINIMISATION_STRICT = 439,
+    VAR_IP_FREEBIND = 440,
+    VAR_DEFINE_TAG = 441,
+    VAR_LOCAL_ZONE_TAG = 442,
+    VAR_ACCESS_CONTROL_TAG = 443,
+    VAR_LOCAL_ZONE_OVERRIDE = 444,
+    VAR_ACCESS_CONTROL_TAG_ACTION = 445,
+    VAR_ACCESS_CONTROL_TAG_DATA = 446,
+    VAR_VIEW = 447,
+    VAR_ACCESS_CONTROL_VIEW = 448,
+    VAR_VIEW_FIRST = 449,
+    VAR_SERVE_EXPIRED = 450,
+    VAR_FAKE_DSA = 451,
+    VAR_FAKE_SHA1 = 452,
+    VAR_LOG_IDENTITY = 453,
+    VAR_HIDE_TRUSTANCHOR = 454,
+    VAR_TRUST_ANCHOR_SIGNALING = 455,
+    VAR_USE_SYSTEMD = 456,
+    VAR_SHM_ENABLE = 457,
+    VAR_SHM_KEY = 458,
+    VAR_DNSCRYPT = 459,
+    VAR_DNSCRYPT_ENABLE = 460,
+    VAR_DNSCRYPT_PORT = 461,
+    VAR_DNSCRYPT_PROVIDER = 462,
+    VAR_DNSCRYPT_SECRET_KEY = 463,
+    VAR_DNSCRYPT_PROVIDER_CERT = 464,
+    VAR_IPSECMOD_ENABLED = 465,
+    VAR_IPSECMOD_HOOK = 466,
+    VAR_IPSECMOD_IGNORE_BOGUS = 467,
+    VAR_IPSECMOD_MAX_TTL = 468,
+    VAR_IPSECMOD_WHITELIST = 469,
+    VAR_IPSECMOD_STRICT = 470
   };
 #endif
 /* Tokens.  */
@@ -390,49 +416,75 @@ extern int yydebug;
 #define VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES 410
 #define VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES 411
 #define VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES 412
-#define VAR_HARDEN_ALGO_DOWNGRADE 413
-#define VAR_IP_TRANSPARENT 414
-#define VAR_DISABLE_DNSSEC_LAME_CHECK 415
-#define VAR_IP_RATELIMIT 416
-#define VAR_IP_RATELIMIT_SLABS 417
-#define VAR_IP_RATELIMIT_SIZE 418
-#define VAR_RATELIMIT 419
-#define VAR_RATELIMIT_SLABS 420
-#define VAR_RATELIMIT_SIZE 421
-#define VAR_RATELIMIT_FOR_DOMAIN 422
-#define VAR_RATELIMIT_BELOW_DOMAIN 423
-#define VAR_IP_RATELIMIT_FACTOR 424
-#define VAR_RATELIMIT_FACTOR 425
-#define VAR_CAPS_WHITELIST 426
-#define VAR_CACHE_MAX_NEGATIVE_TTL 427
-#define VAR_PERMIT_SMALL_HOLDDOWN 428
-#define VAR_QNAME_MINIMISATION 429
-#define VAR_QNAME_MINIMISATION_STRICT 430
-#define VAR_IP_FREEBIND 431
-#define VAR_DEFINE_TAG 432
-#define VAR_LOCAL_ZONE_TAG 433
-#define VAR_ACCESS_CONTROL_TAG 434
-#define VAR_LOCAL_ZONE_OVERRIDE 435
-#define VAR_ACCESS_CONTROL_TAG_ACTION 436
-#define VAR_ACCESS_CONTROL_TAG_DATA 437
-#define VAR_VIEW 438
-#define VAR_ACCESS_CONTROL_VIEW 439
-#define VAR_VIEW_FIRST 440
-#define VAR_SERVE_EXPIRED 441
-#define VAR_FAKE_DSA 442
-#define VAR_LOG_IDENTITY 443
-#define VAR_USE_SYSTEMD 444
+#define VAR_RESPONSE_IP_TAG 413
+#define VAR_RESPONSE_IP 414
+#define VAR_RESPONSE_IP_DATA 415
+#define VAR_HARDEN_ALGO_DOWNGRADE 416
+#define VAR_IP_TRANSPARENT 417
+#define VAR_DISABLE_DNSSEC_LAME_CHECK 418
+#define VAR_IP_RATELIMIT 419
+#define VAR_IP_RATELIMIT_SLABS 420
+#define VAR_IP_RATELIMIT_SIZE 421
+#define VAR_RATELIMIT 422
+#define VAR_RATELIMIT_SLABS 423
+#define VAR_RATELIMIT_SIZE 424
+#define VAR_RATELIMIT_FOR_DOMAIN 425
+#define VAR_RATELIMIT_BELOW_DOMAIN 426
+#define VAR_IP_RATELIMIT_FACTOR 427
+#define VAR_RATELIMIT_FACTOR 428
+#define VAR_SEND_CLIENT_SUBNET 429
+#define VAR_CLIENT_SUBNET_ZONE 430
+#define VAR_CLIENT_SUBNET_ALWAYS_FORWARD 431
+#define VAR_CLIENT_SUBNET_OPCODE 432
+#define VAR_MAX_CLIENT_SUBNET_IPV4 433
+#define VAR_MAX_CLIENT_SUBNET_IPV6 434
+#define VAR_CAPS_WHITELIST 435
+#define VAR_CACHE_MAX_NEGATIVE_TTL 436
+#define VAR_PERMIT_SMALL_HOLDDOWN 437
+#define VAR_QNAME_MINIMISATION 438
+#define VAR_QNAME_MINIMISATION_STRICT 439
+#define VAR_IP_FREEBIND 440
+#define VAR_DEFINE_TAG 441
+#define VAR_LOCAL_ZONE_TAG 442
+#define VAR_ACCESS_CONTROL_TAG 443
+#define VAR_LOCAL_ZONE_OVERRIDE 444
+#define VAR_ACCESS_CONTROL_TAG_ACTION 445
+#define VAR_ACCESS_CONTROL_TAG_DATA 446
+#define VAR_VIEW 447
+#define VAR_ACCESS_CONTROL_VIEW 448
+#define VAR_VIEW_FIRST 449
+#define VAR_SERVE_EXPIRED 450
+#define VAR_FAKE_DSA 451
+#define VAR_FAKE_SHA1 452
+#define VAR_LOG_IDENTITY 453
+#define VAR_HIDE_TRUSTANCHOR 454
+#define VAR_TRUST_ANCHOR_SIGNALING 455
+#define VAR_USE_SYSTEMD 456
+#define VAR_SHM_ENABLE 457
+#define VAR_SHM_KEY 458
+#define VAR_DNSCRYPT 459
+#define VAR_DNSCRYPT_ENABLE 460
+#define VAR_DNSCRYPT_PORT 461
+#define VAR_DNSCRYPT_PROVIDER 462
+#define VAR_DNSCRYPT_SECRET_KEY 463
+#define VAR_DNSCRYPT_PROVIDER_CERT 464
+#define VAR_IPSECMOD_ENABLED 465
+#define VAR_IPSECMOD_HOOK 466
+#define VAR_IPSECMOD_IGNORE_BOGUS 467
+#define VAR_IPSECMOD_MAX_TTL 468
+#define VAR_IPSECMOD_WHITELIST 469
+#define VAR_IPSECMOD_STRICT 470
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 
 union YYSTYPE
 {
-#line 64 "util/configparser.y" /* yacc.c:1909  */
+#line 66 "util/configparser.y" /* yacc.c:1909  */
 
 	char*	str;
 
-#line 436 "util/configparser.h" /* yacc.c:1909  */
+#line 488 "util/configparser.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
